@@ -1,35 +1,11 @@
 #!/usr/bin/env python
 
-# Copyright (c) 2019, IRIS-HEP
-# All rights reserved.
-# 
-# Redistribution and use in source and binary forms, with or without
-# modification, are permitted provided that the following conditions are met:
-# 
-# * Redistributions of source code must retain the above copyright notice, this
-#   list of conditions and the following disclaimer.
-# 
-# * Redistributions in binary form must reproduce the above copyright notice,
-#   this list of conditions and the following disclaimer in the documentation
-#   and/or other materials provided with the distribution.
-# 
-# * Neither the name of the copyright holder nor the names of its
-#   contributors may be used to endorse or promote products derived from
-#   this software without specific prior written permission.
-# 
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
-# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
-# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
-# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+# BSD 3-Clause License; see https://github.com/scikit-hep/awkward-array/blob/master/LICENSE
 
+import functools
 import math
 import numbers
+import operator
 import os
 from collections import OrderedDict
 try:
@@ -765,9 +741,6 @@ class JaggedArray(awkward.array.base.AwkwardArrayWithContent):
         else:
             raise TypeError("invalid index for assigning column to Table: {0}".format(where))
 
-    def _broadcast(self, data):
-        return self.tojagged(data)
-
     def tojagged(self, data):
         if isinstance(data, JaggedArray):
             selfcounts = self.stops - self._starts
@@ -809,9 +782,6 @@ class JaggedArray(awkward.array.base.AwkwardArrayWithContent):
         elif isinstance(data, Iterable):
             return self.tojagged(self.numpy.array(data))
 
-        else:
-            return self.tojagged(self.numpy.array([data]))
-
     def _tojagged(self, starts=None, stops=None, copy=True):
         if starts is None and stops is None:
             if copy:
@@ -975,7 +945,7 @@ class JaggedArray(awkward.array.base.AwkwardArrayWithContent):
 
         counts = stops - starts
         if isinstance(result, tuple):
-            return tuple(self.Methods.maybemixin(type(x), self.JaggedArray).fromcounts(counts, x) if isinstance(x, (self.numpy.ndarray, awkward.array.base.AwkwardBase)) else x for x in result)
+            return tuple(self.Methods.maybemixin(type(x), self.JaggedArray).fromcounts(counts, x) if isinstance(x, (self.numpy.ndarray, awkward.array.base.AwkwardArray)) else x for x in result)
         elif method == "at":
             return None
         else:
@@ -1291,7 +1261,34 @@ class JaggedArray(awkward.array.base.AwkwardArrayWithContent):
 
     def _canuseoffset(self):
         self._valid()
-        return self.offsetsaliased(self._starts, self._stops) or (len(self._starts.shape) == 1 and self.numpy.array_equal(self._starts[1:], self._stops[:-1]))
+        return self.offsetsaliased(self._starts, self._stops) or (len(self._starts.shape) == 1 and self.numpy.array_equal(self._starts[1:], self.stops[:-1]))
+
+    @property
+    def iscompact(self):
+        if len(self._starts) == 0:
+            return True
+        else:
+            flatstarts = self._starts.reshape(-1)
+            flatstops = self.stops.reshape(-1)   # no underscore!
+            if not self.offsetsaliased(self._starts, self._stops) and not self.numpy.array_equal(flatstarts[1:], flatstops[:-1]):
+                return False
+            if not self._isvalid and not (flatstops >= flatstarts).all():
+                raise ValueError("offsets must be monatonically increasing")
+            return True
+
+    def compact(self):
+        if self.iscompact:
+            return self
+        else:
+            offsets = self.counts2offsets(self.counts.reshape(-1))
+            if len(self._starts.shape) == 1:
+                tmp = self
+            else:                                                # no underscore!
+                tmp = self.JaggedArray(self._starts.reshape(-1), self.stops.reshape(-1), self._content)
+            out = tmp._tojagged(offsets[:-1], offsets[1:], copy=False)
+            out.starts.shape = self._starts.shape
+            out.stops.shape = self._starts.shape  # intentional: self._stops can too long
+            return out
 
     @property
     def iscompact(self):
@@ -1340,6 +1337,31 @@ class JaggedArray(awkward.array.base.AwkwardArrayWithContent):
                 out = self.JaggedArray(self._starts.reshape(-1), self._stops.reshape(-1), self._content)
             return out._tojagged(offsets[:-1], offsets[1:], copy=False)._content
 
+    def structure1d(self, levellimit=None):
+        if len(self._starts) > len(self._stops):
+            raise ValueError("starts must have the same (or shorter) length than stops")
+        if self._starts.shape[1:] != self._stops.shape[1:]:
+            raise ValueError("starts and stops must have the same dimensionality (shape[1:])")
+
+        if levellimit is None:
+            levellimit = -1
+
+        out = self
+        if levellimit != 0 and isinstance(self._content, awkward.array.base.AwkwardArray):
+            out = self.copy(content=self._content.structure1d(levellimit - 1))
+
+        while len(out._starts.shape) > 1:
+            last = out._starts.shape[-1]
+            length = functools.reduce(operator.mul, out._starts.shape[:-1], 1)
+            offsets = self.numpy.arange(0, (length + 1)*last, last)
+            outerstarts = offsets[:-1].reshape(out._starts.shape[:-2] + (-1,))
+            outerstops = offsets[1:].reshape(out._stops.shape[:-2] + (-1,))
+            innerstarts = out._starts.reshape(-1)
+            innerstops = out._stops.reshape(-1)
+            out = self.copy(starts=outerstarts, stops=outerstops, content=self.copy(starts=innerstarts, stops=innerstops, content=out._content))
+
+        return out
+
     def _hasjagged(self):
         return True
 
